const width = document.body.clientWidth;
const height = document.body.clientHeight;
const margin = ({top: 10, right: 120, bottom: 10, left: 100});
const dx = 20;
const dy = width / 6;
const tree = d3.tree().nodeSize([dx, dy]);
const diagonal = d3.linkHorizontal()
										.x(d => d.y)
										.y(d => d.x);

d3.json('flare.json')
  .then(data => {
  const root = d3.hierarchy(data);
  root.x0 = dy / 2;
  root.y0 = 0;
  root.descendants().forEach((d, i) => {
    d.id = i;
    d._children = d.children;
    if (d.depth && d.data.name.length !== 7) d.children = null;
  });
    
  const svg = d3.select("svg")
  				.attr('width', width)
  				.attr('height', height)
          .attr("viewBox", [-margin.left, -margin.top, width, dx])
          .style("font", "10px sans-serif")
          .style("user-select", "none")
        .call(d3.zoom()
            // .extent([[0, 0], [width, height]])
            // .scaleExtent([1, 8])
            .on("zoom", ({transform}) => 
                svg.attr("transform", transform)))
        .append("g");

  const gLink = svg.append("g")
      .attr("fill", "#555")
      .attr("stroke", "#555")
      .attr("stroke-opacity", 0.4)
      .attr("stroke-width", 1.5);

  const gNode = svg.append("g")
      .attr("cursor", "pointer")
      .attr("pointer-events", "all");

  function update(source) {
    const duration = d3.event && d3.event.altKey ? 2500 : 250;
    var nodes = root.descendants().reverse();
    var links = root.links();
    // console.log(nodes);

    // Compute the new tree layout.
    tree(root);
    
    var positionDict = {};
    for (var i = 0; i < nodes.length; i++) {
      const name = nodes[i].data.name;
      if ((name in positionDict) && (nodes[i].y <= positionDict[name].y)) {continue}
      else {positionDict[name] = {x: nodes[i].x, y:nodes[i].y}}
		}
    for (var i = 0; i < nodes.length; i++) {
    	const name = nodes[i].data.name;
      if (nodes[i].x !== positionDict[name].x) {
        nodes[i].x = positionDict[name].x + i * 2;
        nodes[i].y = positionDict[name].y + i * 2;
        nodes[i].x0 = nodes[i].x;
        nodes[i].y0 = nodes[i].y;
      }
    }
    // console.log(positionDict);
    // console.log(nodes);

    let left = root;
    let right = root;
    root.eachBefore(node => {
      if (node.x < left.x) left = node;
      if (node.x > right.x) right = node;
    });

    const height = right.x - left.x + margin.top + margin.bottom;

    const transition = svg.transition()
        .duration(duration)
        .attr("viewBox", [-margin.left, left.x - margin.top, width, height])
        .tween("resize", window.ResizeObserver ? null : () => () => svg.dispatch("toggle"));

    // Update the nodes…
    const node = gNode.selectAll("g")
      .data(nodes, d => d.id);
    
    // Enter any new nodes at the parent's previous position.
    const nodeEnter = node.enter().append("g")
        .attr("transform", d => `translate(${source.y0},${source.x0})`)
        .attr("fill-opacity", 0)
        .attr("stroke-opacity", 0)
    		// .each(d => console.log(d.data));
    		// .attr("pointer-events", d =>
    		// clickableNames.has(d.data.name) ? "all" : "none");
    
    // console.log(nodeEnter);
    
    // console.log("nodeEnter: ", nodeEnter.data());
    // console.log("nodes before: ", nodes);
            
    nodeEnter
        .on("click", (event, d) => {
      		console.log("clicked: " + d.data.name + ", parent: " + (d.parent ? d.parent.data.name: "no"));
      		console.log("original", d);
          d.children = d.children ? null : d._children;
      		update(d);
      
      		nodes.forEach(dd => {
            if (dd.data.name === d.data.name && dd !== d) {
              dd.children = dd.children ? null : dd._children;
              update(dd);
            }
            
            else if (dd.children) {
            dd.children.forEach(c => {
              if (c.data.name === d.data.name && d.parent !== dd) {
                console.log("clicked: " + c.data.name + ", parent: " + c.parent.data.name);
                console.log("after", c);
                console.log(c.chilren);
              	c.children = c.children ? null : c._children;
                update(c);
                // console.log("d", d);
                // console.log("parent", c, d.parent == dd);
              }
            })
          }
          })
      
//       		console.log("nodes inside: ", nodes);
        });
    
// 		var clickedNode = null;
//     nodes.forEach(d => {
//       if (d.clicked) {
//         clickedNode = d;
//       }
//     });
    
//     if (clickedNode) {
//       nodes.forEach(d => {
//         if (d.data.name === clickedNode.data.name) {
//           d.clicked = false;
//           d.children = clickedNode.children;
//         }
//       })
//     }
    
//     console.log("nodes after: ", nodes);
    
    // nodeEnter
    // 	.on("click", (event, d) => {
    //   console.log("clicked (nodeEnter): " + d.data.name + ", parent: " + d.parent.data.name);
    //   d.children = d.children ? null: d._children;
    //   update(d);
    //   // nodes.forEach(dd => {
    //   //   if (dd.id !== d.id && dd.data.name === d.data.name) {
    //   //     console.log("clicked (change children): " + dd.data.name + ", parent: " + dd.parent.data.name);
    //   //     dd.children = dd.children ? null: dd._children;
    //   //     update(dd);
    //   //   }
    //   // });
    //   // nodes.forEach(dd => {
    //   //   if (dd.data.name === d.data.name) {
    //   //     console.log("clicked (update): " + dd.data.name + ", parent: " + dd.parent.data.name);
    //   //     update(dd);
    //   //   }
    //   // });
    //   // nodeEnter.filter(dd => dd.data.name == d.data.name)
    //   // 	.each(dd => {dd.children = dd.children ? null : dd._children; 
    //   //                //console.log(dd.data.name + " " + dd.parent.data.name);
    //   //               })
    //   // 	.each(dd => {update(dd)})
    // });

    nodeEnter.append("circle")
        .attr("r", 2.5)
        .attr("fill", d => d._children ? "#555" : "#999")
        .attr("stroke-width", 10);

    nodeEnter.append("text")
        .attr("dy", "0.31em")
        // .attr("x", d => d._children ? -6 : 6)
        // .attr("text-anchor", d => d._children ? "end" : "start")
        .attr("x", -6)
        .attr("text-anchor", "end")
        .text(d => d.data.name)
      .clone(true).lower()
        .attr("stroke-linejoin", "round")
        .attr("stroke-width", 3)
        .attr("stroke", "white");

    // Transition nodes to their new position.
    const nodeUpdate = node.merge(nodeEnter);
    // nodeUpdate
    // 	.each(d => console.log(d.data.name));
    // nodeUpdate
    // 	.on("click", (event, d) => {
    //     nodeUpdate
    //       .filter(dd => dd.data.name === d.data.name)
    //       .each(dd => {
    //         console.log(dd.data.name + " " + dd.parent.data.name);
    //         dd.children = dd.children ? null : dd._children;
    //         update(dd);
    //       })
    // 	})
    nodeUpdate
    	.transition(transition)
        .attr("transform", d => `translate(${d.y},${d.x})`)
        .attr("fill-opacity", 1)
        .attr("stroke-opacity", 1);

    // Transition exiting nodes to the parent's new position.
    const nodeExit = node.exit().transition(transition).remove()
        .attr("transform", d => `translate(${source.y},${source.x})`)
        .attr("fill-opacity", 0)
        .attr("stroke-opacity", 0);

    // Update the links…
    const link = gLink.selectAll("path")
      .data(links, d => (d.source.id, d.target.id));
    	// .data(links, d => (d.source.data.name, d.target.data.name));

    // Enter any new links at the parent's previous position.
    const linkEnter = link.enter().append("path")
        .attr("d", d => {
          const o = {x: source.x0, y: source.y0};
          return diagonal({source: o, target: o});
        });

    // Transition links to their new position.
    link.merge(linkEnter).transition(transition)
        .attr("d", diagonal);

    // Transition exiting nodes to the parent's new position.
    link.exit().transition(transition).remove()
        .attr("d", d => {
          const o = {x: source.x, y: source.y};
          return diagonal({source: o, target: o});
        });

    // Stash the old positions for transition.
    root.eachBefore(d => {
      d.x0 = d.x;
      d.y0 = d.y;
    });
  }

  update(root);

  return svg.node();
})